﻿using System;
using System.ComponentModel;
using System.Windows.Controls;
using C4F.VistaP2P.Common;
using System.Windows;
using System.Windows.Threading;
using System.Threading;

namespace C4F.VistaP2P.WPF.Media
{
    /// <summary>
    /// Interaction logic for Media.xaml
    /// </summary>
    public partial class MediaControl : UserControl
    {
        /// <summary>
        /// Default, autogenerated constructor that shows the Media control
        /// </summary>
        public MediaControl()
        {
            InitializeComponent();            
        }


        /// <summary>
        /// This hooks up the default controls to the properties if custom controls were not set at design time.
        /// </summary>
        /// <param name="sender">unused</param>
        /// <param name="e">unused</param>
        public void MediaControl_Loaded(object sender, EventArgs e)
        {
            if (Title == null)
            {
                Title = lblTitle;
            }            

            if (SendMediaButton == null)
            {
                SendMediaButton = btnSendMedia;
            }

            //if (CancelMediaButton == null)
            //{
            //    CancelMediaButton = btnCancelMedia;
            //}

            if (LineOneMetaData == null)
            {
                LineOneMetaData = lblLineOne;
            }
            if (LineTwoMetaData == null)
            {
                LineTwoMetaData = lblLineTwo;
            }

            if (LineThreeMetaData == null)
            {
                LineThreeMetaData = lblLineThree;
            }

            if (LineOneMetaDataValue == null)
            {
                LineOneMetaDataValue = lblLineOneValue;
            }
            if (LineTwoMetaDataValue == null)
            {
                LineTwoMetaDataValue = lblLineTwoValue;
            }
            if (LineThreeMetaDataValue == null)
            {
                LineThreeMetaDataValue = lblLineThreeValue;
            }

            if (MediaElement == null)
            {
                MediaElement = mPlayer;
            }

        }


#region public methods and events to use when completely overriding this control

        /// <summary>
        /// register for this event to receive messages when the media network status changes 
        /// </summary>
        public event EventHandler<StatusChangedEventArgs> StatusChanged;
        
        /// <summary>
        /// register for this event to know when the media player starts to play a new stream.
        /// </summary>
        public event EventHandler StartPlayChanged;
#endregion

        #region Dependency Properties

        /// <summary>
        /// The dependency property for the Title control. The dependency property is used so we can have databinding using the smart tag adorner
        /// </summary>
        public static DependencyProperty TitleProperty = DependencyProperty.Register(
           "Title", typeof(Label), typeof(MediaControl));

        /// <summary>
        /// Gets or sets the name of the control in the title.
        /// </summary>
        public Label Title
        {
            get
            {
                try
                {
                    return (Label)this.Dispatcher.Invoke(
                       System.Windows.Threading.DispatcherPriority.Normal,
                       (DispatcherOperationCallback)delegate { return GetValue(TitleProperty); },
                       TitleProperty);
                }
                catch
                {

                    return (Label)TitleProperty.DefaultMetadata.DefaultValue;
                }
            }
            set
            {
                Dispatcher.Invoke(DispatcherPriority.Normal,
                  (SendOrPostCallback)delegate { SetValue(TitleProperty, value); },
                  value);
            }
        }

        /// <summary>
        /// The dependency property for used for sending a new media stream. The dependency property is used so we can have databinding using the smart tag adorner
        /// </summary>
        public static DependencyProperty SendMediaButtonProperty = DependencyProperty.Register(
          "SendMediaButton", typeof(Button), typeof(MediaControl), new PropertyMetadata(null, new PropertyChangedCallback(OnSendMediaButtonChanged)));

        /// <summary>
        /// Gets or sets the control to use as the 'SendMediaButton'
        /// </summary>
        public Button SendMediaButton
        {
            get
            {
                try
                {
                    return (Button)this.Dispatcher.Invoke(
                       System.Windows.Threading.DispatcherPriority.Normal,
                       (DispatcherOperationCallback)delegate { return GetValue(SendMediaButtonProperty); },
                       SendMediaButtonProperty);
                }
                catch
                {

                    return (Button)SendMediaButtonProperty.DefaultMetadata.DefaultValue;
                }
            }
            set
            {
                Dispatcher.Invoke(DispatcherPriority.Normal,
                  (SendOrPostCallback)delegate { SetValue(SendMediaButtonProperty, value); },
                  value);
            }
        }

        /// <summary>
        /// This function is called when the SendMediaButton is changed. It unregisters from the old button click
        /// and registers for the new button click.
        /// </summary>
        /// <param name="d">the media control</param>
        /// <param name="e">contains the old and new control</param>
        private static void OnSendMediaButtonChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            MediaControl nm = (MediaControl)d;
            if (e.OldValue != null)
            {
                //unregister from the old one...
                Button oldButton = (Button)e.NewValue;
                oldButton.Click -= new RoutedEventHandler(nm._SendButton_Click);
            }
            //register for the new events.
            Button newButton = (Button)e.NewValue;
            newButton.Click += new RoutedEventHandler(nm._SendButton_Click);
        }



        /// <summary>
        /// The dependency property for used for canceling the media stream. The dependency property is used so we can have databinding using the smart tag adorner
        /// </summary>
        public static DependencyProperty CancelMediaButtonProperty = DependencyProperty.Register(
          "CancelMediaButton", typeof(Button), typeof(MediaControl), new PropertyMetadata(null, new PropertyChangedCallback(OnCancelMediaButtonChanged)));

        /// <summary>
        /// Gets or sets the control to use as the 'CancelMediaButton'
        /// </summary>
        public Button CancelMediaButton
        {
            get
            {
                try
                {
                    return (Button)this.Dispatcher.Invoke(
                       System.Windows.Threading.DispatcherPriority.Normal,
                       (DispatcherOperationCallback)delegate { return GetValue(CancelMediaButtonProperty); },
                       CancelMediaButtonProperty);
                }
                catch
                {

                    return (Button)CancelMediaButtonProperty.DefaultMetadata.DefaultValue;
                }
            }
            set
            {
                Dispatcher.Invoke(DispatcherPriority.Normal,
                  (SendOrPostCallback)delegate { SetValue(CancelMediaButtonProperty, value); },
                  value);
            }
        }


        /// <summary>
        /// This function is called when the SendMediaButton is changed. It unregisters from the old button click
        /// and registers for the new button click.
        /// </summary>
        /// <param name="d">the media control</param>
        /// <param name="e">contains the old and new control</param>
        private static void OnCancelMediaButtonChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            MediaControl nm = (MediaControl)d;
            if (e.OldValue != null)
            {
                //unregister from the old one...
                Button oldButton = (Button)e.NewValue;
                oldButton.Click -= new RoutedEventHandler(nm._CancelButton_Click);
            }
            //register for the new events.
            Button newButton = (Button)e.NewValue;
            newButton.Click += new RoutedEventHandler(nm._CancelButton_Click);
        }


        /// <summary>
        /// The dependency property for used for sending a new media element. The dependency property is used so we can have databinding using the smart tag adorner
        /// </summary>
        public static DependencyProperty MediaElementProperty = DependencyProperty.Register(
            "MediaElement", typeof(MediaElement), typeof(MediaControl));

        /// <summary>
        /// Gets or sets the control to use as the MediaElement
        /// </summary>
        public MediaElement MediaElement
        {
            get
            {
                try
                {
                    return (MediaElement)this.Dispatcher.Invoke(
                       System.Windows.Threading.DispatcherPriority.Normal,
                       (DispatcherOperationCallback)delegate { return GetValue(MediaElementProperty); },
                       MediaElementProperty);
                }
                catch
                {
                    return (MediaElement)MediaElementProperty.DefaultMetadata.DefaultValue;
                }
            }
            set
            {
                Dispatcher.Invoke(DispatcherPriority.Normal,
                  (SendOrPostCallback)delegate { SetValue(MediaElementProperty, value); },
                  value);
            }
        }

        /// <summary>
        /// The dependency property used for the description of the first line of metadata. This is usually the title stored in the file. The dependency property is used so we can have databinding using the smart tag adorner
        /// </summary>
        public static DependencyProperty LineOneMetaDataProperty = DependencyProperty.Register(
           "LineOneMetaData", typeof(Label), typeof(MediaControl));

        /// <summary>
        /// Gets or sets the Label to use for the description of first line of metadata
        /// </summary>
        public Label LineOneMetaData
        {
            get
            {
                try
                {
                    return (Label)this.Dispatcher.Invoke(
                       System.Windows.Threading.DispatcherPriority.Normal,
                       (DispatcherOperationCallback)delegate { return GetValue(LineOneMetaDataProperty); },
                       LineOneMetaDataProperty);
                }
                catch
                {

                    return (Label)LineOneMetaDataProperty.DefaultMetadata.DefaultValue;
                }
            }
            set
            {
                Dispatcher.Invoke(DispatcherPriority.Normal,
                  (SendOrPostCallback)delegate { SetValue(LineOneMetaDataProperty, value); },
                  value);
            }
        }

        /// <summary>
        /// The dependency property used for the description of the second line of metadata. This is usually the album or filename stored in the file. The dependency property is used so we can have databinding using the smart tag adorner
        /// </summary>
        public static DependencyProperty LineTwoMetaDataProperty = DependencyProperty.Register(
           "LineTwoMetaData", typeof(Label), typeof(MediaControl));



        /// <summary>
        /// Gets or sets the Label to use for the description of the second line of metadata
        /// </summary>
        public Label LineTwoMetaData
        {
            get
            {
                try
                {
                    return (Label)this.Dispatcher.Invoke(
                       System.Windows.Threading.DispatcherPriority.Normal,
                       (DispatcherOperationCallback)delegate { return GetValue(LineTwoMetaDataProperty); },
                       LineTwoMetaDataProperty);
                }
                catch
                {

                    return (Label)LineTwoMetaDataProperty.DefaultMetadata.DefaultValue;
                }
            }
            set
            {
                Dispatcher.Invoke(DispatcherPriority.Normal,
                  (SendOrPostCallback)delegate { SetValue(LineTwoMetaDataProperty, value); },
                  value);
            }
        }

        /// <summary>
        /// The dependency property used for the description of the third line of metadata. This is usually the album or filename stored in the file. The dependency property is used so we can have databinding using the smart tag adorner
        /// </summary>
        public static DependencyProperty LineThreeMetaDataProperty = DependencyProperty.Register(
        "LineThreeMetaData", typeof(Label), typeof(MediaControl));

        /// <summary>
        /// Gets or sets the Label to use for the description of the second line of metadata
        /// </summary>
        public Label LineThreeMetaData
        {
            get
            {
                try
                {
                    return (Label)this.Dispatcher.Invoke(
                       System.Windows.Threading.DispatcherPriority.Normal,
                       (DispatcherOperationCallback)delegate { return GetValue(LineThreeMetaDataProperty); },
                       LineThreeMetaDataProperty);
                }
                catch
                {

                    return (Label)LineThreeMetaDataProperty.DefaultMetadata.DefaultValue;
                }
            }
            set
            {
                Dispatcher.Invoke(DispatcherPriority.Normal,
                  (SendOrPostCallback)delegate { SetValue(LineThreeMetaDataProperty, value); },
                  value);
            }
        }


        /// <summary>
        /// The dependency property used for the first line of metadata. This is usually the album or filename stored in the file. The dependency property is used so we can have databinding using the smart tag adorner
        /// </summary>
        public static DependencyProperty LineOneMetaDataValueProperty = DependencyProperty.Register(
          "LineOneMetaDataValue", typeof(Label), typeof(MediaControl));

        /// <summary>
        /// Gets or sets the description of the first line of metadata
        /// </summary>
        public Label LineOneMetaDataValue
        {
            get
            {
                try
                {
                    return (Label)this.Dispatcher.Invoke(
                       System.Windows.Threading.DispatcherPriority.Normal,
                       (DispatcherOperationCallback)delegate { return GetValue(LineOneMetaDataValueProperty); },
                       LineOneMetaDataValueProperty);
                }
                catch
                {

                    return (Label)LineOneMetaDataValueProperty.DefaultMetadata.DefaultValue;
                }
            }
            set
            {
                Dispatcher.Invoke(DispatcherPriority.Normal,
                  (SendOrPostCallback)delegate { SetValue(LineOneMetaDataValueProperty, value); },
                  value);
            }
        }

        /// <summary>
        /// The dependency property used for the second line of metadata. This is usually the album  value or filename stored in the file. The dependency property is used so we can have databinding using the smart tag adorner
        /// </summary>
        public static DependencyProperty LineTwoMetaDataValueProperty = DependencyProperty.Register(
            "LineTwoMetaDataValue", typeof(Label), typeof(MediaControl));

        /// <summary>
        /// Gets or sets the description of the second line of metadata
        /// </summary>
        public Label LineTwoMetaDataValue
        {
            get
            {
                try
                {
                    return (Label)this.Dispatcher.Invoke(
                       System.Windows.Threading.DispatcherPriority.Normal,
                       (DispatcherOperationCallback)delegate { return GetValue(LineTwoMetaDataValueProperty); },
                       LineTwoMetaDataValueProperty);
                }
                catch
                {

                    return (Label)LineTwoMetaDataValueProperty.DefaultMetadata.DefaultValue;
                }
            }
            set
            {
                Dispatcher.Invoke(DispatcherPriority.Normal,
                  (SendOrPostCallback)delegate { SetValue(LineTwoMetaDataValueProperty, value); },
                  value);
            }
        }

        /// <summary>
        /// The dependency property used for the third line of metadata. The dependency property is used so we can have databinding using the smart tag adorner
        /// </summary>
        public static DependencyProperty LineThreeMetaDataValueProperty = DependencyProperty.Register(
            "LineThreeMetaDataValue", typeof(Label), typeof(MediaControl));

        /// <summary>
        /// Gets or sets the description of the third line of metadata
        /// </summary>
        public Label LineThreeMetaDataValue
        {
            get
            {
                try
                {
                    return (Label)this.Dispatcher.Invoke(
                       System.Windows.Threading.DispatcherPriority.Normal,
                       (DispatcherOperationCallback)delegate { return GetValue(LineThreeMetaDataValueProperty); },
                       LineThreeMetaDataValueProperty);
                }
                catch
                {

                    return (Label)LineThreeMetaDataValueProperty.DefaultMetadata.DefaultValue;
                }
            }
            set
            {
                Dispatcher.Invoke(DispatcherPriority.Normal,
                  (SendOrPostCallback)delegate { SetValue(LineThreeMetaDataValueProperty, value); },
                  value);
            }
        }




        #endregion

        /// <summary>
        /// Starts playing the media found at the URI passed in
        /// </summary>
        /// <param name="play">the path the the URI</param>
        public void StartPlay(string play)
        {
            MediaElement.Source = new Uri(play);
            MediaElement.Play();
            if (StartPlayChanged != null)
            {
                StartPlayChanged(this, new EventArgs());
            }
        }


        /// <summary>
        /// saves if this media player is set up for audio or video
        /// </summary>
        private bool mIsMediaVideo;

        /// <summary>
        /// set this media player to be for audio or video.
        /// </summary>
        /// <param name="val"></param>
        internal void IsMediaVideo(bool val)
        {
            mIsMediaVideo = val;
            if (mIsMediaVideo)
            {
                lblTitle.Content = "Video";
            }
            else
            {
                lblTitle.Content = "Audio";
            }
        }

        /// <summary>
        /// The NetworkManager control subscribes to this event so it can send a file to the mesh.
        /// </summary>
        internal event EventHandler<SendStreamEventArgs> SendingStream;
        
        /// <summary>
        /// listens for the media button click.
        /// </summary>
        /// <param name="sender">unused</param>
        /// <param name="e">unused</param>
        private void _SendButton_Click(object sender, EventArgs e)
        {            
            if (SendingStream != null)
            {                
                Microsoft.Win32.OpenFileDialog ofd;
                ofd = new Microsoft.Win32.OpenFileDialog();
                ofd.FileName = "";
                if (mIsMediaVideo)
                {
                    ofd.Filter = "(*.wmv)|*.wmv|All Files (*.*)|*.*";
                    ofd.Title = "Select a Video source";
                }
                else
                {
                    ofd.Filter = "(*.wma)|*.wma|(*.mp3)|*.mp3|All Files (*.*)|*.*";
                    ofd.Title = "Select an Audio source";
                }
                ofd.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                ofd.RestoreDirectory = true;
                
                //check return value, but how in wpf?
                bool ok = (bool)ofd.ShowDialog();
                if (ok)
                {
                    SendingStream(this, new SendStreamEventArgs(ofd.FileName));
                }
            }
        }

        /// <summary>
        /// The NetworkManager control subscribes to this event so it can send a file to the mesh.
        /// </summary>
        internal event EventHandler CancelStream;

        /// <summary>
        /// listens for the cancel 
        /// </summary>
        /// <param name="sender">unused</param>
        /// <param name="e">unused</param>
        private void _CancelButton_Click(object sender, EventArgs e)
        {
            if (CancelStream != null)
            {
                CancelStream(this, new EventArgs());
            }
        }

        private delegate void SetStatusMessageHandler(string msg);
        internal void SetStatusMessage(string msg)
        {
            if (tbStatusLog.CheckAccess())
            {
                this.tbStatusLog.Text += msg + "\r\n";
                tbStatusLog.ScrollToEnd();
            }
            else
            {
                tbStatusLog.Dispatcher.Invoke(DispatcherPriority.Normal, new SetStatusMessageHandler(SetStatusMessage), msg);
            }            
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {

        }
       
        /// <summary>
        /// _StatusChangedHandler delegate used to ensure that the control is only changed by the thread that created it.
        /// look for crossthreading issues for more information.
        /// </summary>
        /// <param name="userName">The username of the NetworkManager tha this Chat control is connected to</param>
        /// <param name="e">The Status changed Event</param>
        private delegate void _SetStatusChangedHandler(string userName, StatusChangedEventArgs e);
        /// <summary>
        /// enables and disables the buttons depending on how the network changed. It also notifies anyone registered for the StatusChanged event.
        /// </summary>
        /// <param name="userName">the network manager's username</param>
        /// <param name="e">contains information on how the network changed.</param>
        internal void SetStatusChanged(string userName, C4F.VistaP2P.Common.StatusChangedEventArgs e)
        {
            if (SendMediaButton.CheckAccess())
            {
                //we're on the controls thread...change it if necessary
                if (userName == e.Member && e.NewNodeJoined)
                {
                    SendMediaButton.IsEnabled = true;
                    CancelMediaButton.IsEnabled = true;
                }

                if (userName == e.Member && e.NodeLeft)
                {
                    SendMediaButton.IsEnabled = false;
                    CancelMediaButton.IsEnabled = false;
                }

                //raise the status changed event for anyone listening.
                if (null != StatusChanged)
                {
                    StatusChanged(userName, e);
                }

            }
            else
            {
                //invoke the call on the controls thread.
                SendMediaButton.Dispatcher.Invoke(DispatcherPriority.Normal, new _SetStatusChangedHandler(SetStatusChanged), userName, e);
            }
        }
    }
}
